mod bufferdata;
mod gpustate;
mod rawtexture;
mod texturearray;
mod util;

pub use gpustate::GPUState;
pub use texturearray::Texture;

use self::bufferdata::Vertex;
use cgmath::Matrix4;

/// API correction matrix.
/// cgmath uses OpenGL's matrix format, which
/// needs to be converted to wgpu's matrix format.
#[rustfmt::skip]
const OPENGL_TO_WGPU_MATRIX: Matrix4<f32> = Matrix4::new(
	1.0, 0.0, 0.0, 0.0,
	0.0, 1.0, 0.0, 0.0,
	0.0, 0.0, 0.5, 0.5,
	0.0, 0.0, 0.0, 1.0,
);

// The surface we draw sprites on.
// Every sprite is an instance of this.
//
// These vertices form a rectangle that covers the whole screen.
// Two facts are important to note:
// - This is centered at (0, 0), so scaling doesn't change a sprite's position
// - At scale = 1, this covers the whole screen. Makes scale calculation easier.
//
// Screen coordinates range from -1 to 1, with the origin at the center.
// Texture coordinates range from 0 to 1, with the origin at the top-left
// and (1,1) at the bottom-right.
const SPRITE_MESH_VERTICES: &[Vertex] = &[
	Vertex {
		position: [-1.0, 1.0, 0.0],
		texture_coords: [0.0, 0.0],
	},
	Vertex {
		position: [1.0, 1.0, 0.0],
		texture_coords: [1.0, 0.0],
	},
	Vertex {
		position: [1.0, -1.0, 0.0],
		texture_coords: [1.0, 1.0],
	},
	Vertex {
		position: [-1.0, -1.0, 0.0],
		texture_coords: [0.0, 1.0],
	},
];

const SPRITE_MESH_INDICES: &[u16] = &[0, 3, 2, 0, 2, 1];
